from beartype import beartype
from loguru import logger as log
from rich.pretty import pretty_repr
from rich.progress import Progress, track

from gitlab import Gitlab
from gitlab.exceptions import GitlabGetError, GitlabListError
from gitlab.v4.objects import Group, Issue, Project, ProjectIssue, Snippet


@beartype
def gitlab_login(gitlab_server: str, gitlab_personal_access_token: str) -> Gitlab:
    """Login to GitLab instance.

    Args:
        gitlab_server (str): URL of GitLab instance.
        gitlab_personal_access_token (str): Personal access token for GitLab instance.

    Returns:
        Gitlab: GitLab api instance with logged in server connection.
    """

    api = Gitlab(url=gitlab_server, private_token=gitlab_personal_access_token)
    api.auth()
    log.info(f"Logged in as: {api.user.username} [UID {api.user.id}]")

    return api


@beartype
def get_gitlab_items(
    api: Gitlab,
    gitlab_group_ids: list[int] | None,
    gitlab_project_ids: list[int] | None,
    include_personal_projects: bool = False,
    include_wikis: bool = False,
    include_issues: bool = False,
    include_snippets: bool = False,
) -> tuple[
    dict[int, Group],
    dict[int, Project],
    dict[int, Group | Project],
    dict[int, Issue | ProjectIssue],
    dict[int, Snippet],
]:
    """Get GitLab group and projects as backup items.

    Args:
        api (Gitlab): Logged in GitLab api instance.
        gitlab_group_ids (list[int]): List of GitLab group ids.
        gitlab_project_ids (list[int]): List of GitLab project ids.
        include_personal_projects (bool, optional): Include personal projects in backup. Defaults to False.

    Returns:
        dict[int, Group]: Gitlab groups dictionary.
        dict[int, Project]: Gitlab projects dictionary.
    """

    gitlab_groups = dict()
    gitlab_projects = dict()
    gitlab_wikis = dict()
    if gitlab_group_ids:
        with Progress() as pbar:
            task_1 = pbar.add_task("Searching specified groups...", total=len(gitlab_group_ids))
            for gid in gitlab_group_ids:
                group = api.groups.get(gid)

                if group:
                    gitlab_groups, gitlab_projects = _recurse_group(api, group, gitlab_groups, gitlab_projects, pbar)

                pbar.update(task_1, advance=1)

    if gitlab_project_ids:
        for pid in track(gitlab_project_ids, description="Searching specified projects..."):
            try:
                project = api.projects.get(pid)
            except GitlabGetError:
                log.warning(f"Failed to get project for PID {pid}")

            if project:
                log.info(f"Found project: {project.name} [PID {project.id}]")
                gitlab_projects[project.id] = project
                gitlab_projects.update({project.id: project})

    if include_personal_projects:
        personal_projects = api.users.get(api.user.id).projects.list(all=True)
        for project in track(personal_projects, description="Searching personal projects..."):
            log.info(f"Found user project: {project.name} [PID {project.id}]")
            gitlab_projects.update({project.id: project})

    if include_wikis:
        for group in track(gitlab_groups.values(), description="Searching group wikis..."):
            try:
                if hasattr(group, "wikis") and (pages := group.wikis.list(all=True)):
                    log.info(f"Found wiki with {len(pages)} pages for group {group.name} [GID {group.id}]")
                    gitlab_wikis.update({f"G{group.id}": group})
            except GitlabListError:
                log.warning(f"Failed to get wiki for group {group.name} [GID {group.id}]")

        for project in track(gitlab_projects.values(), description="Searching project wikis..."):
            try:
                if hasattr(project, "wikis") and (pages := project.wikis.list(all=True)):
                    log.info(f"Found wiki with {len(pages)} pages for project {project.name} [PID {project.id}]")
                    gitlab_wikis.update({f"P{project.id}": project})
            except GitlabListError:
                log.warning(f"Failed to get wiki for project {project.name} [GID {project.id}]")

    gitlab_issues = dict()
    if include_issues:
        for pid in track(gitlab_projects.keys(), description="Searching issues..."):
            issues = api.projects.get(pid).issues.list(all=True)
            for issue in issues:
                log.info(f"Found issue: {issue.title} [IID {issue.iid}]")
                gitlab_issues.update({f"{pid}/{issue.iid}": issue})

    gitlab_snippets = dict()
    if include_snippets:
        snippets = api.snippets.list(all=True)
        for snippet in track(snippets, description="Searching snippets..."):
            log.info(f"Found snippet: {snippet.title} [SID {snippet.id}]")
            gitlab_snippets.update({snippet.id: snippet})

    log.debug(f"GitLab groups:\n{pretty_repr(gitlab_groups)}")
    log.debug(f"GitLab projects:\n{pretty_repr(gitlab_projects)}")
    log.debug(f"GitLab issues:\n{pretty_repr(gitlab_issues)}")
    log.debug(f"GitLab snippets:\n{pretty_repr(gitlab_snippets)}")

    return gitlab_groups, gitlab_projects, gitlab_wikis, gitlab_issues, gitlab_snippets


@beartype
def _recurse_group(
    api: Gitlab, group: Group, groups: dict[str, Group], projects: dict[str, Project], pbar: Progress
) -> tuple[dict[str, Group], dict[str, Project]]:
    """Recursively get GitLab subgroups and projects from a group

    Args:
        api (Gitlab): Logged in GitLab api instance.
        groups (list[int]): Gitlab groups dictionary.
        projects (list[int]): Gitlab projects dictionary.
        pbar (Progerss): Rich progress bar instance.

    Returns:
        dict[int, Group]: Gitlab groups dictionary.
        dict[int, Project]: Gitlab projects dictionary.
    """

    log.info(f"Found group: {group.name} [GID {group.id}]")
    groups.update({group.id: group})

    subgroups = group.subgroups.list(all=True)
    if subgroups:
        log.debug(f"Found {len(subgroups)} subgroups in group {group.name} [GID {group.id}]")
        task_11 = pbar.add_task("\tIterating subgroups...", total=len(subgroups))
        for subgroup in subgroups:
            subgroup = api.groups.get(subgroup.id)
            groups, projects = _recurse_group(api, subgroup, groups, projects, pbar)

            pbar.update(task_11, advance=1)

        pbar.remove_task(task_11)
    else:
        log.debug("Max group depth reached.")

    group_projects = group.projects.list(all=True)
    if group_projects:
        log.debug(f"Found {len(group_projects)} in {group.name} [GID {group.id}]")
        task_12 = pbar.add_task("\tIterating group projects...", total=len(group_projects))
        for project in group_projects:
            log.info("Found group project:" f"{project.name} [PID {project.id}] in group {group.name} [GID {group.id}]")
            projects.update({project.id: project})

            pbar.update(task_id=task_12, advance=1)

        pbar.remove_task(task_12)

    return groups, projects



import json
import os
from multiprocessing import Pool
from pathlib import Path

from beartype import beartype
from beartype.typing import Any
from loguru import logger as log

import git
from gitlab.v4.objects import Group, Project, Snippet


@beartype
def clone_project(args) -> dict[str, Any]:
    """Clone a GitLab project to a local path."""
    project: Project
    target_path: str

    result = {
        "project_name": "",
        "project_url": "",
        "project_id": 0,
        "project_path": "",
        "is_cloned": False,
        "submodules": list(),
        "is_submodules_updated": False,
        "branches": list(),
        "is_branches_updated": False,
        "success": False,
    }
    try:
        project, target_path = args

        result.update(
            {
                "project_name": project.name,
                "project_url": project.web_url,
                "project_id": project.id,
                "project_path": target_path,
            }
        )
        log.info(f"Cloning project: {project.name} <{project.web_url}> to {target_path}...")

        repository = None
        try:
            repository = git.Repo.clone_from(
                project.web_url,
                target_path,
            )
            result["is_cloned"] = True
            log.info(f"Cloning {project.name} has completed.")
        except Exception as e:
            log.warning(f"Cloning {project.name} has failed: {e}")

        if repository:
            if repository.submodules:
                try:
                    result["submodules"] = [submodule.url for submodule in repository.submodules]
                    log.info("Update submodules...")
                    are_submodules_updated = len(repository.submodules) * [False]
                    for ids, submodule in enumerate(repository.submodules):
                        try:
                            submodule_path = (Path(target_path) / submodule.path).as_posix()
                            git.Repo.clone_from(submodule.url, submodule_path, recursive=True)
                            are_submodules_updated[ids] = True
                            log.info(f"Cloning submodule {submodule.name} has completed.")
                        except Exception as e:
                            log.warning(f"Updating submodules has failed: {e}")

                    repository.submodule_update(recursive=True)
                    result["is_submodules_cloned"] = all(are_submodules_updated)
                    log.info("Updated submodules.")
                except Exception as e:
                    log.warning(f"Updating submodules has failed: {e}")

            remote_branches = repository.git.branch("--all").replace("remotes/", "").split("\n")[2:]
            remote_branches = [b.strip() for b in remote_branches]
            log.debug(f"Found {len(remote_branches)} remote branches: {remote_branches}")
            result["branches"] = remote_branches
            if len(remote_branches) > 1:
                for remote_branch in remote_branches:
                    try:
                        local_branch = remote_branch.replace("origin/", "").strip()
                        remote_branch = remote_branch.strip()
                        repository.git.branch("--track", local_branch, remote_branch)
                        log.debug(f"Tracking branch {remote_branch}.")
                    except Exception:
                        log.warning(f"Tracking branch {remote_branch} has failed.")

                try:
                    log.info("Pulling all additional branches...")
                    repository.git.fetch("--all")
                    repository.git.pull("--all")

                    result["is_branches_updated"] = True
                except Exception as e:
                    log.warning(f"Pulling all additional branches has failed: {e}")

            else:
                log.info("No additional branches to pull.")
                result["is_branches_updated"] = True

            with open(Path(target_path) / f"../project_{project.id}.json", "w", encoding="utf-8") as f:
                json.dump(project.asdict(), f, indent=4, sort_keys=True)

            result["success"] = (
                True
                if result["is_cloned"]
                and (bool(result["submodules"]) is result["is_submodules_updated"])
                and result["is_branches_updated"]
                else False
            )

    except KeyboardInterrupt:
        log.warning(f"Cloning {project.name} has been interrupted.")
        result["success"] = False
    except Exception as e:
        log.error(f"Cloning {project.name} has failed: {e}")
        result["success"] = False
    finally:
        return result


@beartype
def clone_wiki(args) -> dict[str, Any]:
    """Clone a GitLab project to a local path."""
    obj: Group | Project
    target_path: str

    result = {
        "wiki_name": "",
        "wiki_url": "",
        "wiki_id": 0,
        "wiki_path": "",
        "is_cloned": False,
        "submodules": list(),
        "is_submodules_updated": False,
        "branches": list(),
        "is_branches_updated": False,
        "success": False,
    }
    try:
        obj, target_path = args
        result.update(
            {
                "wiki_name": obj.name + " Wiki",
                "wiki_url": obj.web_url + ".wiki.git",
                "wiki_id": obj.id,
                "wiki_path": target_path,
            }
        )
        log.info(f"Cloning wiki: {result['wiki_name']} <{result['wiki_url']}> to {target_path}...")

        repository = None
        try:
            repository = git.Repo.clone_from(
                result["wiki_url"],
                target_path,
            )
            result["is_cloned"] = True
            log.info(f"Cloning {result['wiki_name']} has completed.")
        except Exception as e:
            log.warning(f"Cloning {result['wiki_name']} has failed: {e}")

        if repository:
            if repository.submodules:
                try:
                    result["submodules"] = [submodule.url for submodule in repository.submodules]
                    are_submodules_updated = len(repository.submodules) * [False]
                    for ids, submodule in enumerate(repository.submodules):
                        try:
                            submodule_path = (Path(target_path) / submodule.path).as_posix()
                            git.Repo.clone_from(submodule.url, submodule_path, recursive=True)
                            are_submodules_updated[ids] = True
                            log.info(f"Cloning submodule {submodule.name} has completed.")
                        except Exception as e:
                            log.warning(f"Updating submodules has failed: {e}")

                    repository.submodule_update(recursive=True)
                    result["is_submodules_cloned"] = all(are_submodules_updated)
                    log.info("Updated submodules.")
                except Exception as e:
                    log.warning(f"Updating submodules has failed: {e}")

            remote_branches = repository.git.branch("--all").replace("remotes/", "").split("\n")[2:]
            remote_branches = [b.strip() for b in remote_branches]
            log.debug(f"Found {len(remote_branches)} remote branches: {remote_branches}")
            result["branches"] = remote_branches
            if len(remote_branches) > 1:
                for remote_branch in remote_branches:
                    try:
                        local_branch = remote_branch.replace("origin/", "").strip()
                        remote_branch = remote_branch.strip()
                        repository.git.branch("--track", local_branch, remote_branch)
                        log.debug(f"Tracking branch {remote_branch}.")
                    except Exception:
                        log.warning(f"Tracking branch {remote_branch} has failed.")

                try:
                    log.info("Pulling all additional branches...")
                    repository.git.fetch("--all")
                    repository.git.pull("--all")

                    result["is_branches_updated"] = True
                except Exception as e:
                    log.warning(f"Pulling all additional branches has failed: {e}")

            else:
                log.info("No additional branches to pull.")
                result["is_branches_updated"] = True

            result["success"] = (
                True
                if result["is_cloned"]
                and (bool(result["submodules"]) is result["is_submodules_updated"])
                and result["is_branches_updated"]
                else False
            )

    except KeyboardInterrupt:
        log.warning(f"Cloning {result['wiki_name']} has been interrupted.")
        result["success"] = False
    except Exception as e:
        log.error(f"Cloning {result['wiki_name']} has failed: {e}")
        result["success"] = False
    finally:
        return result


@beartype
def clone_snippet(args) -> dict[str, Any]:
    """Clone a GitLab snippet to a local path."""
    snippet: Snippet
    target_path: str

    result = {
        "snippet_title": "",
        "snippet_url": "",
        "snippet_id": 0,
        "snippet_path": "",
        "is_cloned": False,
        "success": False,
    }
    try:
        snippet, target_path = args

        result.update(
            {
                "snippet_title": snippet.title,
                "snippet_url": snippet.web_url,
                "snippet_id": snippet.id,
                "snippet_path": target_path,
            }
        )
        log.info(f"Cloning snippet: {snippet.id} <{snippet.web_url}> to {target_path}...")

        repository = None
        try:
            repository = git.Repo.clone_from(snippet.web_url.replace("/-/", "/"), target_path)
            result["is_cloned"] = True
            log.info(f"Cloning {snippet.id} has completed.")
        except Exception as e:
            log.warning(f"Cloning {snippet.id} has failed: {e}")

        if repository:
            with open(Path(target_path) / f"../snippet_{snippet.id}.json", "w", encoding="utf-8") as f:
                json.dump(snippet.asdict(), f, indent=4, sort_keys=True)

            result["success"] = True if result["is_cloned"] else False

    except KeyboardInterrupt:
        log.warning(f"Cloning {snippet.id} has been interrupted.")
        result["success"] = False
    except Exception as e:
        log.error(f"Cloning {snippet.id} has failed: {e}")
        result["success"] = False
    finally:
        return result


@beartype
def clone_projects(projects: dict[str, Project], backup_dir: Path) -> None:
    """Clone GitLab projects to a local path.

    Args:
        projects (dict[str, Project]): GitLab projects dictionary.
        backup_dir (Path): Backup directory.
    """
    mapped_args = list()
    for project in projects.values():
        mapped_args.append((project, Path(backup_dir / project.path_with_namespace).resolve().as_posix()))

    with Pool(processes=min(1, os.cpu_count() - 2)) as pool:
        results = pool.map(clone_project, mapped_args)

    success = all(list(map(lambda x: x["success"], results)))
    log.info(f"Cloning {'has' if success else 'has not'} been completed successfully.")
    log.info(f"Successful clones: {list(map(lambda x: x['project_name'], filter(lambda x: x['success'], results)))}")
    if not success:
        log.warning(
            f"Failed clones: {list(map(lambda x: x['project_name'], filter(lambda x: not x['success'], results)))}"
        )

    log.info("See results.json for details.")
    with open(backup_dir / "results.json", "w", encoding="utf-8") as f:
        json.dump(results, f, indent=4, sort_keys=True)


@beartype
def clone_wikis(obj: dict[str, Group | Project], backup_dir: Path) -> None:
    """Clone GitLab group or project wiki to a local path.

    Args:
        groups (dict[str, Group | Project]): GitLab groups dictionary.
        backup_dir (Path): Backup directory.
    """
    mapped_args = list()
    for obj in obj.values():
        mapped_args.append((obj, Path(backup_dir / f"{obj.path_with_namespace}.wiki").resolve().as_posix()))

    with Pool(processes=min(1, os.cpu_count() - 2)) as pool:
        results = pool.map(clone_wiki, mapped_args)

    success = all(list(map(lambda x: x["success"], results)))
    log.info(f"Cloning {'has' if success else 'has not'} been completed successfully.")
    log.info(f"Successful clones: {list(map(lambda x: x['wiki_name'], filter(lambda x: x['success'], results)))}")
    if not success:
        log.warning(
            f"Failed clones: {list(map(lambda x: x['wiki_name'], filter(lambda x: not x['success'], results)))}"
        )

    log.info("See results.json for details.")
    with open(backup_dir / "results.json", "w", encoding="utf-8") as f:
        json.dump(results, f, indent=4, sort_keys=True)


@beartype
def clone_snippets(snippets: dict[int, Snippet], backup_dir: Path) -> None:
    """Clone GitLab snippets to a local path.

    Args:
        snippets (dict[int, Snippet]): GitLab snippets dictionary.
        backup_dir (Path):  Backup directory.
    """
    mapped_args = list()
    backup_dir = Path(backup_dir / "snippets")
    backup_dir.mkdir(parents=True, exist_ok=True)
    for snippet in snippets.values():
        mapped_args.append((snippet, Path(backup_dir / f"{snippet.id}").resolve().as_posix()))

    with Pool(processes=min(1, os.cpu_count() - 2)) as pool:
        results = pool.map(clone_snippet, mapped_args)

    success = all(list(map(lambda x: x["success"], results)))
    log.info(f"Cloning {'has' if success else 'has not'} been completed successfully.")
    log.info(f"Successful clones: {list(map(lambda x: x['snippet_id'], filter(lambda x: x['success'], results)))}")
    if not success:
        log.warning(
            f"Failed clones: {list(map(lambda x: x['snippet_id'], filter(lambda x: not x['success'], results)))}"
        )

    log.info("See results.json for details.")
    with open(backup_dir / "results.json", "w", encoding="utf-8") as f:
        json.dump(results, f, indent=4, sort_keys=True)



import json
import os
import re
import time
from pathlib import Path

import requests
from beartype import beartype
from beartype.typing import Any, List
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.chrome.service import Service as ChromeService
from selenium.webdriver.common.by import By
from webdriver_manager.chrome import ChromeDriverManager

from gitlab.v4.objects import Group, Issue, Project, ProjectIssueNote

# Regex pattern to match strings starting with 'uploads/'
FILE_UPLOAD_PATTERN = r"uploads\/[a-zA-Z0-9\/_.-]+"


@beartype
def download_file_from_url(
    server: str, username: str, password: str, file_urls: List[str], download_dir: str = ""
) -> None:
    """Downloads files from given URLs using Selenium and ChromeDriver, after logging in with provided credentials.

    Args:
        server (str): The base URL of the server where the files are hosted.
        username (str): The username for login.
        password (str): The password for login.
        attachment_urls (List[str]): A list of URLs (relative to the server base URL) for the attachments to download.
        download_dir (str, optional): The directory where files should be downloaded. Defaults to the current directory.
    """
    chrome_options = Options()
    if download_dir:
        # Ensure the download directory exists
        download_dir = Path(download_dir).resolve()
        download_dir.mkdir(parents=True, exist_ok=True)
        # Set Chrome preferences for downloads
        chrome_prefs = {
            "download.default_directory": str(download_dir),
            "download.prompt_for_download": False,  # Disable download prompt
            "download.directory_upgrade": True,  # Enable directory upgrade
            "safebrowsing_for_trusted_sources_enabled": False,  # Disable safe browsing for trusted sources
            "safebrowsing.enabled": False,  # Disable safe browsing
            "plugins.always_open_pdf_externally": True,  # Automatically download PDFs instead of opening them
        }
        chrome_options.add_experimental_option("prefs", chrome_prefs)

    # Initialize the Chrome WebDriver
    driver = webdriver.Chrome(service=ChromeService(ChromeDriverManager().install()), options=chrome_options)

    # Perform login
    login_url = f"{server}/users/sign_in"
    driver.get(login_url)
    time.sleep(2)  # Wait for the login page to load

    # Fill in the login credentials and submit
    driver.find_element(By.ID, "user_login").send_keys(username)
    time.sleep(1)
    driver.find_element(By.ID, "user_password").send_keys(password)
    time.sleep(1)
    driver.find_element(By.CLASS_NAME, "js-sign-in-button").click()
    time.sleep(3)  # Wait for login to complete

    # Download each file by navigating to its URL
    for file_url in file_urls:
        if file_url.endswith((".png", ".jpg", ".jpeg", ".gif", ".html")):
            selenium_cookies = driver.get_cookies()
            cookies = {cookie["name"]: cookie["value"] for cookie in selenium_cookies}

            local_filename = file_url.split("/")[-1]
            i = 0
            while (download_dir / local_filename).exists():
                i += 1
                local_filename = Path(local_filename).stem + f"_{i}" + Path(local_filename).suffix

            with requests.get(file_url, cookies=cookies, stream=True) as r:
                r.raise_for_status()
                with open(download_dir / local_filename, "wb") as f:
                    for chunk in r.iter_content(chunk_size=8192):
                        f.write(chunk)

        else:
            driver.get(file_url)

        time.sleep(2)  # Wait for each file to start downloading

    # Placeholder for user to check downloads before quitting
    driver.quit()


@beartype
def find_matches(obj: None | str | list | dict[str, Any], pattern: str, found: List[str] = None) -> List[str]:
    """
    Recursively searches for all strings in a nested JSON-like object that match a given regex pattern.

    Args:
        obj (dict[str, Any]): The JSON-like object (dict, list, or str) to search through.
        pattern (str): The regex pattern to match strings against.
        found (List[str], optional): Accumulator for matches found during recursion. Defaults to None.

    Returns:
        List[str]: A list of all strings found in the object that match the regex pattern.
    """
    if found is None:
        found = []

    if isinstance(obj, dict):
        for value in obj.values():
            find_matches(value, pattern, found)

    elif isinstance(obj, list):
        for item in obj:
            find_matches(item, pattern, found)

    elif isinstance(obj, str):
        found += re.findall(pattern, obj)

    return found


@beartype
def get_wiki_attachment_urls(obj: Group | Project, pattern: str) -> List[str]:
    """Extracts attachment URLs from the "description" field of a GitLab wiki objects.

    Args:
        obj (Group | Project): The wiki object to extract attachment URLs from.
        pattern (str): The regex pattern to match strings against.

    Returns:
        List[str]: A list of attachment URLs extracted from the wiki contents.
    """

    pages = obj.wikis.list(all=True)
    matches = []
    for page in pages:
        content = obj.wikis.get(page.slug).content
        matches += find_matches(content, pattern)

    attachment_urls = []
    for attachment_uri in matches:
        if not attachment_uri:
            continue

        if "uploads/-" in attachment_uri:
            attachment_uri = attachment_uri.replace("uploads/-/", "-/wikis/uploads/")

        url = obj.web_url
        attachment_url = f"{url}/-/wikis/{attachment_uri}"
        attachment_urls.append(attachment_url)

    return attachment_urls


def get_issue_attachment_urls(
    obj: Issue | ProjectIssueNote, pattern: str, is_note=False, web_url: str = ""
) -> List[str]:
    """Extracts attachment URLs from the "description" field of a GitLab issue.

    Args:
        issue (dict[str, Any]): The issue json object to extract attachment URLs from.
        pattern (str): The regex pattern to match strings against.

    Returns:
        List[str]: A list of attachment URLs extracted from the issue description.
    """
    if is_note:
        matches = find_matches(obj.body, pattern)
    else:
        matches = find_matches(obj.description, pattern)

    attachment_urls = []
    for attachment_uri in matches:
        if "uploads/-" in attachment_uri:
            attachment_uri = attachment_uri.replace("uploads/-/", "uploads/")

        url = obj.web_url.split("/-/issues/")[0] if not is_note else web_url.split("/-/issues/")[0]
        attachment_url = f"{url}/{attachment_uri}"
        attachment_urls.append(attachment_url)

    return attachment_urls


if __name__ == "__main__":
    from dotenv import load_dotenv

    load_dotenv()

    # Your GitLab credentials and URL
    SERVER = os.getenv("GITLAB_SERVER")
    USERNAME = os.getenv("GITLAB_USERNAME")
    PASSWORD = os.getenv("GITLAB_PASSWORD")

    # Load the JSON data from a file
    with open("EXAMPLE_PATH_TO_ISSUE_JSON") as f:
        issue = json.load(f)

    # Assuming you have a function `download_file_from_url` defined elsewhere
    # from your_module import download_file_from_url
    attachment_urls = get_issue_attachment_urls(issue, FILE_UPLOAD_PATTERN)

    # Assuming `SERVER`, `USERNAME`, `PASSWORD`, and `download_dir` are defined
    download_file_from_url(SERVER, USERNAME, PASSWORD, attachment_urls, download_dir="./")



import os
import sys
from pathlib import Path

import hydra
from beartype import beartype
from hydra.core.hydra_config import HydraConfig
from loguru import logger as log
from omegaconf import DictConfig
from rich.pretty import pretty_repr

from foxysafe import CONFIG_DIR, CONFIG_NAME
from foxysafe.gitlab.backups import gitlab_backup_routine
from foxysafe.utils.custom_logging import disable_logging_groups, setup_custom_hydra_logging


@hydra.main(CONFIG_DIR.as_posix(), CONFIG_NAME, version_base="1.3")
@beartype
def backup_routine_entrypoint(config: DictConfig):
    """Entrypoint for foxysafe backup routing configured by hydra YAML config file.

    Args:
        config (DictConfig): Hydra config object.
    """

    setup_custom_hydra_logging(config)

    log.info(f"Found {CONFIG_NAME} configuration file in {CONFIG_DIR}")
    log.debug(f"Configuration:\n{pretty_repr(config)}")

    hydra_internal_config = HydraConfig.get()

    backup_dir = (Path(hydra_internal_config.runtime.output_dir) / "foxysafe.backup/").resolve()
    backup_dir.mkdir(parents=True, exist_ok=True)
    os.chdir(backup_dir)

    gitlab_backup_routine(config, backup_dir)


def main():
    """Entrypoint for foxysafe configured by hydra YAML config file."""

    disable_logging_groups("urllib3")

    try:
        hydra_working_dir = "./foxysafe_out/${now:%Y-%m-%d}/${now:%H-%M-%S}/"
        sys.argv += [f"hydra.run.dir={hydra_working_dir}"]

        backup_routine_entrypoint()
    except KeyboardInterrupt:
        log.info("Exiting...")
        exit(0)
    except Exception:
        log.exception("An unhandled runtime exception occurred.")
        exit(1)


if __name__ == "__main__":
    main()



from pathlib import Path
from warnings import filterwarnings

from beartype.roar import BeartypeDecorHintPep585DeprecationWarning
from dotenv import load_dotenv
from rich.traceback import install as rich_traceback_install

from foxysafe import custom_hydra_plugins

CONFIG_DIR = (Path(__file__).parent.parent).resolve()
CONFIG_NAME = "default_config"

__all__ = ["custom_hydra_plugins", "CONFIG_DIR", "CONFIG_NAME"]

load_dotenv((CONFIG_DIR / ".env").as_posix())

filterwarnings("ignore", category=BeartypeDecorHintPep585DeprecationWarning)

rich_traceback_install()
